/**
 * 预览效果的本地服务及子应用打包webpack的配置
 * @author yanrong.tian
 * @date 2020/5/16 14:22:00
 */

const path = require('path');
const fs = require('fs');
const HtmlWebPackPlugin = require('html-webpack-plugin');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const CssMinimizerPlugin = require('css-minimizer-webpack-plugin');
const lessLoader = require.resolve('less-loader');
const TerserPlugin = require('terser-webpack-plugin');
const webpack = require('webpack');
// const webpackDevMiddleware = require('webpack-dev-middleware');
const ModuleFederationPlugin = require('webpack/lib/container/ModuleFederationPlugin');
const BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin;
const MonacoWebpackPlugin = require('monaco-editor-webpack-plugin');
const AntdDayjsWebpackPlugin = require('antd-dayjs-webpack-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const OfflinePlugin = require('@lcdp/offline-plugin');
const Webpack = require('webpack');
const {alias} = require('./alias');

const { getBabelLoaderConfig, getStyleLoaders } = require('./common');
const lessModuleRegex = /\.module\.(scss|less)$/;

exports.Webpack = Webpack;

// exports.webpackDevMiddleware = webpackDevMiddleware;

exports.getWebpackConfig = ({
  layoutRoot,
  projectRoot, // 项目根目录
  publicPath,
  webpackConfig,
  outputPath,
  lessVariables = {},
  devServer,
  entry, // 入口
  templatePath, // 模板路径
  requirePath, // 发送请求的文件路径, 控制请求拦载
  command,
  isBundleAnalyzerPlugin,
  isPxToRem = false,
}) => {
  let config = {
    mode: 'production',

    entry: entry,

    devtool: false,

    output: {
      path: outputPath,
      filename: '[name][hash].js',
      publicPath: publicPath,
      chunkFilename: '[name][hash].js',
    },

    devServer: devServer || {},

    resolve: {
      extensions: ['.js', '.jsx', '.ts', '.tsx', '.json', '.mdx'],
    },

    optimization: {
      splitChunks: {
        chunks: 'async',
        minSize: 50000,
        maxSize: 1000000,
        minChunks: 1,
        maxAsyncRequests: 6,
        maxInitialRequests: 1,
        automaticNameDelimiter: '~',
        maxAsyncSize: 60000000,
        cacheGroups: {
          monacoEditor: {
            chunks: 'async',
            name: 'chunk-monaco-editor',
            priority: 22,
            test: /[\/]node_modules[\/]monaco-editor[\/]/,
            enforce: true,
            reuseExistingChunk: true,
          },
          vendors: {
            name: `chunk-vendors`,
            test: /[\\/]node_modules[\\/]/,
            priority: -10,
            chunks: 'initial',
            reuseExistingChunk: true,
          },
          common: {
            name: `chunk-common`,
            minChunks: 2,
            priority: -20,
            chunks: 'initial',
            reuseExistingChunk: true,
          },
        },
      },
      chunkIds: 'named',
      minimize: command === 'start' ? false : true,
      minimizer:
        command === 'start'
        ? []
        : [
            new CssMinimizerPlugin(),
            '...'
          ],
    },

    module: {
      rules: [
        {
          test: [
            /\.bmp$/,
            /\.gif$/,
            /\.jpe?g$/,
            /\.png$/,
            /\.webp$/,
            /\.svg/,
            /\.ttf/,
            /\.woff/,
            /\.eot/,
            /\.otf/,
            /\.woff2/,
          ],
          loader: require.resolve('url-loader'),
          options: {
            limit: 10000,
            name: '[name].[ext]',
            esModule: false,
          },
        },
        {
          test: /\.(ts|tsx|js|jsx|mjs)$/,
          //exclude: /node_modules/,
          use: [getBabelLoaderConfig()],
        },
        {
          test: /\.css$/, // 解析css
          use: getStyleLoaders(isPxToRem,true, {
            importLoaders: 1,
          }),
        },
        {
          test: /\.less$/, // 解析less
          exclude: lessModuleRegex,
          use: getStyleLoaders(
            isPxToRem,
            true,
            {
              importLoaders: 1,
            },
            {
              loader: lessLoader,
              options: {
                lessOptions: {
                  modifyVars: lessVariables,
                  javascriptEnabled: true,
                },

              },
            },

          ),
        },
        {
          test: lessModuleRegex,
          use: getStyleLoaders(
            isPxToRem,
            true,
            {
              importLoaders: 2,
              modules: true,
              localIdentName: '[path][name]__[local]',
              namedExport: true,
              camelCase: true,
            },
            {
              loader: lessLoader,
              options: {
                lessOptions: {
                  modifyVars: lessVariables,
                  javascriptEnabled: true,
                },
              },
            },
          ),
        },
      ],
    },

    plugins: [

      new HtmlWebPackPlugin({
        template: templatePath,
        filename: path.join(outputPath, `/index.html`),
        hash: true, // 会在打包好的bundle.js后面加上类似于: ?6ce91c19b9054fff5a05 串
      }),

      new webpack.DefinePlugin({
        '__REACT_DEVTOOLS_GLOBAL_HOOK__': '({ isDisabled: true })',
        _PROCESS_ENV_PUNLIC_PATH_: '"' + publicPath + '"',
        _PROCESS_ENV_BASE_: '"' + publicPath + '"',
        NODE_ENV: 'production',
      }),

      new MiniCssExtractPlugin({
        filename: '[name][hash].css',
        chunkFilename: 'chunk.[name].[hash].css',
        ignoreOrder: true,
      }),

      // new AntdDayjsWebpackPlugin()

      //new webpack.HotModuleReplacementPlugin(),
    ],
  };

  if (isBundleAnalyzerPlugin) {
    config.plugins.push(new BundleAnalyzerPlugin());
  }

  config = webpackConfig(config, {
    requestPath: requirePath,
    moduleFederationPlugin: ModuleFederationPlugin,
    monacoWebpackPlugin: MonacoWebpackPlugin,
    CopyWebpackPlugin: CopyWebpackPlugin,
    offlinePlugin: OfflinePlugin,
    command,
  });

  config.resolve.alias['@layout'] = layoutRoot;

  const isInGlobal = fs.existsSync(path.join(__dirname, '../node_modules/import-remote-module'));
  if (isInGlobal && command !== 'pack') {
    config.resolve.alias = Object.assign({}, alias, config.resolve.alias);
  }

  return config;
};
